from __future__ import annotations

from typing import Union, TYPE_CHECKING

import O365 as off

from miscutils import lazy_property

from .folder import MessageFolder
from .message import FluentMessage

if TYPE_CHECKING:
    from ..office import Office


class OutlookService:
    """A class representing Microsoft Outlook. Controls access to email-related services."""

    def __init__(self, office: Office) -> None:
        self.office = office
        self.mailbox = Mailbox(parent=self.office.account, main_resource=self.office.account.main_resource, name='MailBox')
        self._signature = self.office.config.appdata.new_file("signature", "html")

    def __getitem__(self, key: Union[str, int]) -> MessageFolder:
        return self.custom(folder_name=key) if isinstance(key, str) else (self.custom(folder_id=key) if isinstance(key, int) else None)

    @property
    def message(self) -> FluentMessage:
        """A property that will create a new fluent message."""
        return FluentMessage(parent=self.main)

    @property
    def signature(self) -> str:
        """A property controlling access to the user's signature. Changes to it will be persisted to the filesystem across sessions."""
        return self._signature.contents

    @signature.setter
    def signature(self, signature: str) -> None:
        self._signature.contents = signature

    @lazy_property
    def main(self) -> MessageFolder:
        """A property that returns the main folder."""
        return MessageFolder(parent=self.office.account, main_resource=self.office.account.main_resource, root=True, name='Main')

    @lazy_property
    def inbox(self) -> MessageFolder:
        """A property that returns the inbox folder."""
        return self.mailbox.inbox_folder()

    @lazy_property
    def outbox(self) -> MessageFolder:
        """A property that returns the outbox folder."""
        return self.mailbox.outbox_folder()

    @lazy_property
    def sent(self) -> MessageFolder:
        """A property that returns the sent folder."""
        return self.mailbox.sent_folder()

    @lazy_property
    def drafts(self) -> MessageFolder:
        """A property that returns the drafts folder."""
        return self.mailbox.drafts_folder()

    @lazy_property
    def junk(self) -> MessageFolder:
        """A property that returns the junk folder."""
        return self.mailbox.junk_folder()

    @lazy_property
    def deleted(self) -> MessageFolder:
        """A property that returns the deleted folder."""
        return self.mailbox.deleted_folder()

    def custom(self, folder_name: str = None, folder_id: int = None) -> MessageFolder:
        """Return the given custom folder by name or id."""
        return self.mailbox.get_folder(folder_name=folder_name, folder_id=folder_id)


class Mailbox(off.mailbox.MailBox):
    folder_constructor = MessageFolder



from __future__ import annotations

from typing import Any
import webbrowser

from maybe import Maybe
from miscutils import Supressor

with Supressor():
    import O365 as off

if True:
    from .config import Config
    from .calendar import CalendarService
    from .outlook import OutlookService
    from .people import PeopleService


class Office:
    """The primary class controlling access to the entirety of this library via its attributes named after Office365 services."""

    scopes = [
        "offline_access", "User.Read", "Mail.ReadWrite", "Mail.Send", "Mail.ReadWrite.Shared", "Mail.Send.Shared",
        "Contacts.ReadWrite", "Contacts.ReadWrite.Shared", "Calendars.ReadWrite", "Calendars.ReadWrite.Shared",
        "Files.ReadWrite", "Files.ReadWrite.All", "Sites.Read.All", "Sites.ReadWrite.All",
    ]

    def __init__(self, email_address: str = None, connection: str = None) -> None:
        self.outlook: OutlookService = None
        self.people: PeopleService = None
        self.calendar: CalendarService = None

        self.config = Config()
        self.connection = Maybe(connection).else_(self.config.data.default_connections.office)
        self.token = off.FileSystemTokenBackend(token_path=str(self.config.appdata), token_filename="o365_token.txt")

        settings = self.config.data.connections.office[self.connection]
        self.address = Maybe(email_address).else_(settings.default_email)
        self.account = Account((settings.id, settings.secret), main_resource=self.address, token_backend=self.token, office=self)

        try:
            self._establish_services()
        except RuntimeError:
            self.request_token()
            self._establish_services()

    def __repr__(self) -> str:
        return f"{type(self).__name__}(account={self.address})"

    def request_token(self) -> None:
        """ """
        auth_url = self.account.connection.get_authorization_url(requested_scopes=self.scopes)
        webbrowser.open(auth_url)
        self.account.connection.request_token(input("Please follow the link that will open momentarily and grant permission. Then enter the url of the inbox page you land on.\n\n"))

    def _establish_services(self) -> None:
        self.outlook, self.people, self.calendar = OutlookService(office=self), PeopleService(office=self), CalendarService(office=self)


class Account(off.Account):
    def __init__(self, *args: Any, office: Office = None, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.con.office = office



from __future__ import annotations

import datetime as dt
from typing import Union, Collection, TYPE_CHECKING

import O365.calendar as calendar

from ..query import Query, BulkAction, BulkActionContext
from ..fluent import FluentEntity

if TYPE_CHECKING:
    from ..people import Contact


class Event(calendar.Event):
    """A class representing a Microsoft Outlook message. Provides methods and properties for interacting with it."""

    def __repr__(self) -> str:
        return f"{type(self).__name__}({', '.join([f'{attr}={repr(val)}' for attr, val in self.__dict__.items() if not attr.startswith('_')])})"

    def __str__(self) -> str:
        return self.text

    @property
    def fluent(self) -> FluentEvent:
        return FluentEvent(parent=self)


class BulkEventAction(BulkAction):
    """A class representing a bulk action performed on the resultset of a folder query."""

    def delete(self) -> BulkActionContext:
        """Delete all events that match the query this bulk action was created from."""
        return BulkActionContext(query=self._query, action=Event.delete)


class EventQuery(Query):
    """A class for querying the message folders within a given collection."""

    @property
    def bulk(self) -> BulkEventAction:
        """Perform a bulk action on the resultset of this query."""
        return BulkEventAction(self)


class FluentEvent(FluentEntity):
    """A class representing an event that doesn't yet exist. All public methods allow chaining. At the end of the method chain call FluentEvent.create() to create the event."""

    def __init__(self, parent: Event = None) -> None:
        self.entity, self.office = parent, parent.con.office
        self._temp_body: str = None

    def from_(self, address: str) -> FluentEvent:
        """Set the email address this event will appear to originate from."""
        self.entity.organizer = address
        return self

    def to(self, contacts: Union[Union[str, Contact], Collection[Union[str, Contact]]]) -> FluentEvent:
        """Set the email address(es) (a single one or a collection of them) this event will be sent to. Email addresses can be provided either as strings or as contact objects."""
        self.entity.attendees.add(self._parse_contacts_to_emails(contacts=contacts))
        return self

    def start(self, start_time: dt.datetime = None) -> FluentEvent:
        """Set the time at which the event will start."""
        self._start = start_time
        return self

    def end(self, end_time: dt.datetime = None) -> FluentEvent:
        """Set the time at which the event will end."""
        self._end = end_time
        return self

    def all_day(self, is_all_day: bool = True) -> FluentEvent:
        self.entity.is_all_day = is_all_day
        return self

    def location(self, location: str) -> FluentEvent:
        self.entity.location = location
        return self

    def remind_before_minutes(self, remind_before_minutes: int) -> FluentEvent:
        self.entity.remind_before_minutes = remind_before_minutes
        return self

    def response_requested(self, response_requested: bool) -> FluentEvent:
        self.entity.response_requested = response_requested
        return self

    def show_as(self, show_as: str) -> FluentEvent:
        self.entity.show_as = show_as
        return self

    def sensitivity(self, sensitivity: str) -> FluentEvent:
        self.entity.sensitivity = sensitivity
        return self

    def create(self) -> bool:
        """Create this event as it currently is."""
        if self._temp_body is not None:
            self.entity.body = f"{self._temp_body}<br><br>{self.office.outlook.signature}" if self._signing else self._temp_body

        return self.entity.save()



import pytest


def test_Attribute___init__():
    assert True


def test_Attribute___repr__():
    assert True


def test_BaseAttributeMeta___and__():
    assert True


def test_BaseAttributeMeta___eq__():
    assert True


def test_BaseAttributeMeta___ge__():
    assert True


def test_BaseAttributeMeta___gt__():
    assert True


def test_BaseAttributeMeta___le__():
    assert True


def test_BaseAttributeMeta___lt__():
    assert True


def test_BaseAttributeMeta___ne__():
    assert True


def test_BaseAttributeMeta___or__():
    assert True


def test_BaseAttributeMeta__resolve():
    assert True


def test_BaseAttribute___repr__():
    assert True


def test_BooleanAttributeMeta___and__():
    assert True


def test_BooleanAttributeMeta___eq__():
    assert True


def test_BooleanAttributeMeta___ge__():
    assert True


def test_BooleanAttributeMeta___gt__():
    assert True


def test_BooleanAttributeMeta___invert__():
    assert True


def test_BooleanAttributeMeta___le__():
    assert True


def test_BooleanAttributeMeta___lt__():
    assert True


def test_BooleanAttributeMeta___ne__():
    assert True


def test_BooleanAttributeMeta___or__():
    assert True


def test_BooleanAttributeMeta__resolve():
    assert True


def test_BooleanAttribute___init__():
    assert True


def test_BooleanAttribute___repr__():
    assert True


def test_BooleanExpressionClause___and__():
    assert True


def test_BooleanExpressionClause___init__():
    assert True


def test_BooleanExpressionClause___or__():
    assert True


def test_BooleanExpressionClause__coalesce():
    assert True


def test_BooleanExpressionClause__resolve():
    assert True


def test_BooleanExpression___and__():
    assert True


def test_BooleanExpression___init__():
    assert True


def test_BooleanExpression___invert__():
    assert True


def test_BooleanExpression___or__():
    assert True


def test_BooleanExpression___repr__():
    assert True


def test_BooleanExpression__resolve():
    assert True


def test_BooleanExpression_negate():
    assert True


def test_EnumerativeAttributeMeta___and__():
    assert True


def test_EnumerativeAttributeMeta___eq__():
    assert True


def test_EnumerativeAttributeMeta___ge__():
    assert True


def test_EnumerativeAttributeMeta___gt__():
    assert True


def test_EnumerativeAttributeMeta___le__():
    assert True


def test_EnumerativeAttributeMeta___lt__():
    assert True


def test_EnumerativeAttributeMeta___ne__():
    assert True


def test_EnumerativeAttributeMeta___new__():
    assert True


def test_EnumerativeAttributeMeta___or__():
    assert True


def test_EnumerativeAttributeMeta__resolve():
    assert True


def test_EnumerativeAttribute___init__():
    assert True


def test_EnumerativeAttribute___repr__():
    assert True


def test_FilterableAttribute___init__():
    assert True


def test_FilterableAttribute___repr__():
    assert True


def test_NonFilterableAttribute___repr__():
    assert True


def test_NonFilterableMeta___getattr__():
    assert True



__all__ = ["Office", "CalendarService", "Calendar", "Event", "OutlookService", "MessageFolder", "Message", "PeopleService", "ContactFolder", "Contact", "Config", "BlobStorage"]

from .office import Office
from .calendar import CalendarService, Calendar, Event
from .outlook import OutlookService, MessageFolder, Message
from .people import PeopleService, ContactFolder, Contact
from .config import Config
from .blob import BlobStorage



from __future__ import annotations

import os
from typing import Any, Iterator

from maybe import Maybe
from subtypes import NameSpace
from pathmagic import File, Dir, PathLike

from office import resources
from .config import Config


class BlobStorage:
    """A class representing a blob storage account. Takes a connection alias which must exist in the library config settings. If none is provided, the default connection will be used."""

    def __init__(self, connection: str = None) -> None:
        import azure.storage.blob as blob

        self.config = Config()
        self.connection = Maybe(connection).else_(self.config.data.default_connections.blob)
        settings = self.config.data.connections.blob[self.connection]

        self.blob, self.service = blob, blob.BlockBlobService(account_name=settings.account, account_key=settings.key)
        self.containers = BlobContainerNameSpace(self)
        self.blob_type_mappings = File.from_resource(package=resources, name="blob_content_types", extension="json").contents


class BlobContainerNameSpace(NameSpace):
    """A namespace class representing a collection of blob containers."""

    def __init__(self, blob_manager: BlobStorage) -> None:
        self._manager = blob_manager
        super().__init__({container.name: BlobContainer(container=container, blob_manager=self._manager) for container in self._manager.service.list_containers()})


class BlobContainer:
    """A class representing a blob container, with methods for accessing (supports item access) and iterating over its blobs."""

    def __init__(self, container: Any, blob_manager: BlobStorage) -> None:
        self.raw, self.name, self.manager = container, container.name, blob_manager
        self.service = self.manager.service
        self._cached_len = 0

    def __repr__(self) -> str:
        return f"{type(self).__name__}(name={repr(self.name)}, num_blobs={repr(self._cached_len or '?')})"

    def __str__(self) -> str:
        return self.name

    def __len__(self) -> int:
        self._cached_len = sum([1 for blob in self.service.list_blob_names(self.name)])
        return self._cached_len

    def __bool__(self) -> bool:
        return len(self) > 0

    def __iter__(self) -> Iterator[Blob]:
        return iter(Blob(blob=name, container=self) for name in self.service.list_blobs(self.name))

    def __getitem__(self, key: str) -> Blob:
        return Blob(blob=self.service.get_blob_properties(container_name=self.name, blob_name=key), container=self)

    def download_blob_to(self, blob_name: str, path: PathLike) -> PathLike:
        """Download the named blob to the given folder. It will keep its blob 'basename' as its new name."""
        return self[blob_name].download_to(path)

    def download_blob_to_path(self, blob_name: str, path: PathLike) -> PathLike:
        """Download the named blob to the given path."""
        return self[blob_name].download_to_path(path)

    def upload_blob_from(self, blob_name: str, path: PathLike) -> Blob:
        """Create a new blob within this container in storage from the given file path."""
        from azure.storage.blob.models import ContentSettings

        file = File(path)
        content_type = self.manager.blob_type_mappings[file.extension]
        self.service.create_blob_from_path(container_name=self.name, blob_name=blob_name, file_path=str(file), content_settings=ContentSettings(content_type=content_type))

        return self[blob_name]


class Blob:
    """A class representing a blob in storage within a container, with methods for interacting with it."""

    def __init__(self, blob: Any, container: BlobContainer) -> None:
        self.raw, self.name, self.container = blob, blob.name, container
        self.service = self.container.manager.service

    def __repr__(self) -> str:
        return f"{type(self).__name__}(name={repr(self.name)}, container={repr(self.container.name)})"

    def download_to(self, path: PathLike) -> File:
        """Download this blob to the given folder. It will keep its blob 'basename' as its new name."""
        file = Dir(path).new_file(self.name)
        self.service.get_blob_to_path(container_name=self.container.name, blob_name=self.name, file_path=str(file))
        return file

    def download_to_path(self, path: PathLike) -> File:
        """Download this blob to the given path."""
        self.service.get_blob_to_path(container_name=self.container.name, blob_name=self.name, file_path=os.fspath(path))
        return File.from_pathlike(path)

    def delete(self) -> None:
        """Permanently delete this blob within its container in storage."""
        self.service.delete_blob(container_name=self.container.name, blob_name=self.name)



from __future__ import annotations

from typing import List

import O365.address_book as address_book

from .contact import Contact, ContactQuery
from ..attribute import Attribute, NonFilterableAttribute
from ..query import Query, BulkAction, BulkActionContext
from ..outlook import Message


class ContactFolder(address_book.ContactFolder):
    """A class representing a Microsoft People contact folder. Can initiate queries on any contacts or folders it contains."""

    message_constructor, contact_constructor = Message, Contact

    @property
    def folders(self) -> ContactFolderQuery:
        """A property that will create a new query against the folders contained within this folder."""
        return ContactFolderQuery(container=self)

    @property
    def contacts(self) -> ContactQuery:
        """A property that will create a new query against the contacts contained within this folder."""
        return ContactQuery(container=self)

    def from_address(self, address: str) -> Contact:
        """Return the contact with the given address if one exists. Otherwise return None."""
        return self.get_contact_by_email(address)

    class Attributes:
        class Name(Attribute):
            name = "display_name"

        class Contacts(NonFilterableAttribute):
            name = "contacts"

        class ChildFolders(NonFilterableAttribute):
            name = "child_folders"


class BulkContactFolderAction(BulkAction):
    """A class representing a bulk action performed on the resultset of a contact folder query."""

    def move(self, folder: ContactFolder) -> BulkActionContext:
        """Move all folders that match the query this bulk action was created from into the given folder."""
        return BulkActionContext(query=self._query, action=ContactFolder.move_folder, args=(folder,))

    def delete(self) -> BulkActionContext:
        """Delete all folders that match the query this bulk action was created from."""
        return BulkActionContext(query=self._query, action=ContactFolder.delete)


class ContactFolderQuery(Query):
    """A class for querying the contact folders within a given collection."""

    def __getitem__(self, key: str) -> ContactFolder:
        return self._container.get_folder(folder_name=key)

    def execute(self) -> List[Contact]:
        """Execute this query and return any folders that match."""
        return list(self._container.get_folders(limit=self._limit, query=self._query))

    @property
    def bulk(self) -> BulkContactFolderAction:
        """Perform a bulk action on the resultset of this query."""
        return BulkContactFolderAction(self)



from setuptools import setup, find_packages
from os import path

__version__ = "0.1.0"

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="office365",
    version=__version__,
    description="A wrapper around O365 offering subclasses with additional utility methods.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/matthewgdv/office",
    license="MIT",
    classifiers=[
      "Development Status :: 3 - Alpha",
      "Intended Audience :: Developers",
      "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(exclude=["tests*"]),
    install_requires=[

    ],
    setup_requires=['setuptools_scm'],
    include_package_data=True,
    author="Matt GdV",
    author_email="matthewgdv@gmail.com"
)
